#include "common/meta.hpp"
#include "common/utils.hpp"
#include "nodes/base/base.hpp"

namespace GNode
{

BaseNode::BaseNode(const std::string& name, NODE_TYPE type)
{
    name_ = name;
    type_ = type;
}

BaseNode::~BaseNode()
{

}

void BaseNode::start()
{
    if (!running_.exchange(true))
    {
        worker_thread_ = std::thread(&BaseNode::work, this);
    }
}

void BaseNode::stop()
{
    if (running_.exchange(false))
    {
        // 删除队列中全部元素
        PLOGI.printf("Node : [%s] delete queue start", name_.c_str());
        std::for_each(input_buffers_.begin(), input_buffers_.end(),
                  [&](const auto &item) { item.second->clear(); });
        std::for_each(output_buffers_.begin(), output_buffers_.end(),
                    [&](const auto &item) { item.second->clear(); });
        PLOGI.printf("Node : [%s] delete queue stop", name_.c_str());
        cond_var_->notify_all();
        if (worker_thread_.joinable())
        {
            worker_thread_.join();
        }
        PLOGI.printf("Node : [%s] stopped", name_.c_str());
    }
}

void BaseNode::work()
{
    while (running_)
    {
        Timer timer(name_);
        bool has_data = false;
        for (auto& input_buffer : input_buffers_)
        {
            std::shared_ptr<meta::MetaData> meta_data;
            if (!input_buffer.second->try_pop(meta_data))
            {
                continue;
            }
            has_data = true;
            handle_data(meta_data);
            send_output_data(meta_data);          
        }
        if (!has_data)
        {
            std::unique_lock<std::mutex> lock(mutex_);
            cond_var_->wait_for(lock, std::chrono::milliseconds(100), [this] {
                return !running_;  // 等待时检查退出条件
            });
        }
    }
}

void BaseNode::handle_data(std::shared_ptr<meta::MetaData>& data)
{
    
}

} // namespace Node